/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* NASA CRM - NASA Common Research Model
 * CAD created by Guillem Battle I Capa
 * Valid mesh obtained with gmsh 4.8.4
 */

/* Model information
 * Length of fuselage: 62.79 m
 * Chord length at wing root: 12.09 m
 * Chord length at Yehudi break: 7.30 m
 * Chord length at wing tip: 2.76 m
 * Chord length at tail root: 5.45 m
 * Chord length at tail tip: 2.33 m
 * Reference wing area: 1/2 * 383.69 m^2
 * Reference chord length: 7.01 m
 */

/// Constants
scl = 2; // scaling factor for lifting surfaces mesh size (testing purpose)
DefineConstant[ msFar = {36.8, Name "Mesh size on farfield" },
                msFus = {0.628, Name "Mesh size on fuselage" },
                msWingR = {0.121 * scl, Name "Mesh size on wing root" },
				msWingY = {0.073 * scl, Name "Mesh size on wing Yehudi break" },
				msWingT = {0.028 * scl, Name "Mesh size on wing tip" },
				msTailR = {0.055 * scl, Name "Mesh size on tail root" },
                msTailT = {0.023 * scl, Name "Mesh size on tail tip" } ];

/// Geometry definition
// Import
Geometry.OCCAutoFix = 1;
Geometry.OCCFixDegenerated = 1;
Geometry.OCCFixSmallEdges = 1;
Geometry.OCCFixSmallFaces = 1;
Geometry.OCCSewFaces = 1;
Geometry.Tolerance = 1e-5; // 0.01 mm
Geometry.OCCTargetUnit = "M";
Merge "crm.stp";
Coherence;

// Re-orient
ReverseMesh Surface{1:48}; //To reverse the direction of the mesh normal vectors

// Volume
Surface Loop(1) = {4:48};
Volume(1) = {1};

// Embedded
Line{3} In Surface{16};
Line{7} In Surface{16};
Line{15} In Surface{16};
Surface{1} In Volume{1};
Surface{2} In Volume{1};
Surface{3} In Volume{1};

// Physical groups
Physical Line("wakeExW") = {10,11,12,13};
Physical Line("teW") = {9,14};
Physical Line("wakeTipW") = {10};
Physical Line("wakeExT") = {1,2,5};
Physical Line("teT") = {6};
Physical Line("wakeTipT") = {1,2};
Physical Surface("symmetry") = {19,20,21,22};
Physical Surface("downstream") = {16};
Physical Surface("upstream") = {18};
Physical Surface("farfield") = {14,15,17};
Physical Surface("fuselage") = {13,23:45};
Physical Surface("wing") = {4,5,6,11,12};
Physical Surface("wing_") = {7,8,9,10};
Physical Surface("tail") = {47,48};
Physical Surface("tail_") = {46};
Physical Surface("wakeW") = {2,3};
Physical Surface("wakeT") = {1};
Physical Volume("field") = {1};

/// Mesh definition
// Farfield
Characteristic Length {34,35,36,37,38,39,40,43,44,45} = msFar; // farfield boundaries
Characteristic Length {3,4,7,8,15,41,42} = msFar; // wake

// Fuselage
Characteristic Length {11,12,13,51,61,50,73,74,62,63,64,65,47,29,70,46} = msFus; // body
Characteristic Length {51,52,53,54,55,56,59,60,89,85,86,90,91,92} = 0.5 * msFus; // nose and windshield
Characteristic Length {57,58,87,88} = 0.2 * msFus; // surface between windshield
Characteristic Length {49,72,28,33,32,24,31,30,75,71,48,76,69,47} = 0.75 * msFus; // wingbox
Characteristic Length {5,66,67,68} = 0.25 * msFus; // APU
Characteristic Length {46,11} = msFus; // Fuselage-wake intersection

// Wing
Characteristic Length {24, 25,26,27,14} = msWingR; // wing root
Characteristic Length {22, 20,21,23,9} = msWingY; // wing kink
Characteristic Length {18,10,16,19} = 2 * msWingT; // wing tip TE
Characteristic Length {17} = msWingT; // wing tip LE

// Tail
Characteristic Length {79:84,93:99, 100:102,77,78,103,6} = msTailR; // Tail root
Characteristic Length {1,2,105} = 2 * msTailT; // tail tip TE
Characteristic Length {104} = msTailT; // tail tip LE

// Mesh algos
Mesh.Algorithm = 1;
Mesh.Algorithm3D = 2;
Mesh.Optimize = 1;
Mesh.Smoothing = 10;
Mesh.SmoothNormals = 1;

Mesh 1;
Geometry.Tolerance = 1e-6;
Coherence Mesh;
