#include "DBoundaryLayer.h"
#include "DClosures.h"
#include "DDiscretization.h"
#include "DEdge.h"
#include <cmath>
#include <iomanip>

using namespace vii;

/**
 * @brief Construct a new BoundaryLayer::BoundaryLayer object.
 *
 * @param _xtrF Forced transition location (default=-1; free transition).
 */
BoundaryLayer::BoundaryLayer(double _xtrF)
{
    if (_xtrF < 0. && _xtrF != -1.)
        throw std::runtime_error("Fatal error: Invalid transition location.\n");

    xtrF = _xtrF;
    xtr = 1.0;
    closSolver = new Closures();
    mesh = new Discretization();
    blEdge = new Edge();
    transMarker = 0;
}

BoundaryLayer::~BoundaryLayer()
{
    delete closSolver;
    delete mesh;
    delete blEdge;
    // std::cout << "~vii::BoundaryLayer()\n";
}

/**
 * @brief Set the mesh and resize all boundary layer quantities accordingly.
 *
 * @param x Position x in the global frame of reference.
 * @param y Position y in the global frame of reference.
 * @param z Position z in the global frame of reference.
 */
void BoundaryLayer::setMesh(std::vector<double> x,
                            std::vector<double> y,
                            std::vector<double> z)
{
    size_t nMarkers = x.size();
    mesh->setGlob(x, y, z);

    // Resize and reset all variables if needed.
    if (mesh->getDiffnElm() != 0)
    {
        u.resize(nVar * nMarkers, 0.);
        Ret.resize(nMarkers, 0.);
        cd.resize(nMarkers, 0.);
        cf.resize(nMarkers, 0.);
        Hstar.resize(nMarkers, 0.);
        Hstar2.resize(nMarkers, 0.);
        Hk.resize(nMarkers, 0.);
        ctEq.resize(nMarkers, 0.);
        us.resize(nMarkers, 0.);
        deltaStar.resize(nMarkers, 0.);
        delta.resize(nMarkers, 0.);
        regime.resize(nMarkers, 0);
    }
}

/**
 * @brief Set the boundary condition at the stagnation point.
 *
 * @param Re Freestream Reynolds number.
 */
void BoundaryLayer::setStagBC(double Re)
{
    double dv0 = (blEdge->getVt(1) - blEdge->getVt(0)) / (mesh->getLoc(1) - mesh->getLoc(0));
    if (dv0 > 0)
        u[0] = sqrt(0.075 / (Re * dv0)); // Theta
    else
        u[0] = 1e-6;         // Theta
    u[1] = 2.23;             // H
    u[2] = 0.;               // N
    u[3] = blEdge->getVt(0); // ue
    u[4] = 0.;               // Ctau

    Ret[0] = u[3] * u[0] * Re;
    if (Ret[0] < 1)
    {
        Ret[0] = 1.;
        u[3] = Ret[0] / (u[0] * Re);
    }
    deltaStar[0] = u[0] * u[1];

    std::vector<double> lamParam(8, 0.);
    closSolver->laminarClosures(lamParam, u[0], u[1], Ret[0], blEdge->getMe(0), name);

    Hstar[0] = lamParam[0];
    Hstar2[0] = lamParam[1];
    Hk[0] = lamParam[2];
    cf[0] = lamParam[3];
    cd[0] = lamParam[4];
    delta[0] = lamParam[5];
    ctEq[0] = lamParam[6];
    us[0] = lamParam[7];
}

/**
 * @brief Set the boundary condition at the first wake point.
 *
 * @param Re Freestream Reynolds number
 * @param UpperCond Variables at the last point on the upper side.
 * @param LowerCond Variables at the last point on the lower side.
 */
void BoundaryLayer::setWakeBC(double Re, std::vector<double> UpperCond, std::vector<double> LowerCond)
{
    if (name != "wake")
        std::cout << "Warning: Imposing wake boundary condition on " << name << std::endl;
    u[0] = UpperCond[0] + LowerCond[0];                                        // (Theta_up+Theta_low).
    u[1] = (UpperCond[0] * UpperCond[1] + LowerCond[0] * LowerCond[1]) / u[0]; // ((Theta*H)_up+(Theta*H)_low)/(Theta_up+Theta_low).
    u[2] = 9.;                                                                 // Amplification factor.
    u[3] = blEdge->getVt(0);                                                   // Inviscid velocity.
    u[4] = (UpperCond[0] * UpperCond[4] + LowerCond[0] * LowerCond[4]) / u[0]; // ((Ct*Theta)_up+(Theta)_low)/(Ct*Theta_up+Theta_low).

    Ret[0] = u[3] * u[0] * Re; // Reynolds number based on the momentum thickness.

    if (Ret[0] < 1)
    {
        Ret[0] = 1;
        u[3] = Ret[0] / (u[0] * Re);
    }
    deltaStar[0] = u[0] * u[1];

    // Laminar closures.
    std::vector<double> lamParam(8, 0.);
    closSolver->laminarClosures(lamParam, u[0], u[1], Ret[0], blEdge->getMe(0), name);
    Hstar[0] = lamParam[0];
    Hstar2[0] = lamParam[1];
    Hk[0] = lamParam[2];
    cf[0] = lamParam[3];
    cd[0] = lamParam[4];
    delta[0] = lamParam[5];
    ctEq[0] = lamParam[6];
    us[0] = lamParam[7];

    for (size_t k = 0; k < mesh->getnMarkers(); ++k)
        regime[k] = 1;
}

/**
 * @brief Compute the blowing velocity in each station of the region.
 *
 */
void BoundaryLayer::computeBlwVel()
{
    deltaStar[0] = u[0] * u[1];
    blowingVelocity.resize(mesh->getnMarkers() - 1, 0.);
    for (size_t iPoint = 1; iPoint < mesh->getnMarkers(); ++iPoint)
    {
        deltaStar[iPoint] = u[iPoint * nVar + 0] * u[iPoint * nVar + 1];
        blowingVelocity[iPoint - 1] = (blEdge->getRhoe(iPoint) * blEdge->getVt(iPoint) * deltaStar[iPoint] - blEdge->getRhoe(iPoint - 1) * blEdge->getVt(iPoint - 1) * deltaStar[iPoint - 1]) / (blEdge->getRhoe(iPoint) * (mesh->getLoc(iPoint) - mesh->getLoc(iPoint - 1)));
    }
}

/**
 * @brief Print solution at a given station.
 *
 * @param iPoint Marker id.
 * @note Function used for debugging.
 */
void BoundaryLayer::printSolution(size_t iPoint) const
{
    std::cout << "Pt " << iPoint << "Reg " << regime[iPoint] << std::endl;
    std::cout << "x " << mesh->getx(iPoint) << "xx " << mesh->getLoc(iPoint) << "xxExt " << mesh->getExt(iPoint) << std::endl;
    std::cout << std::scientific << std::setprecision(15);
    std::cout << std::setw(10) << "T " << u[iPoint * nVar + 0] << "\n"
              << std::setw(10) << "H " << u[iPoint * nVar + 1] << "\n"
              << std::setw(10) << "N " << u[iPoint * nVar + 2] << "\n"
              << std::setw(10) << "ue " << u[iPoint * nVar + 3] << "\n"
              << std::setw(10) << "Ct " << u[iPoint * nVar + 4] << "\n"
              << std::setw(10) << "dStar (BL) " << deltaStar[iPoint] << "\n"
              << std::setw(10) << "dStar (old) " << blEdge->getDeltaStar(iPoint) << "\n"
              << std::setw(10) << "vt " << blEdge->getVt(iPoint) << "\n"
              << std::setw(10) << "Me " << blEdge->getMe(iPoint) << "\n"
              << std::setw(10) << "rhoe " << blEdge->getRhoe(iPoint) << std::endl;
}