#ifndef DDRIVER_H
#define DDRIVER_H

#include "vii.h"
#include "wObject.h"
#include "DBoundaryLayer.h"
#include "wTimers.h"
namespace vii
{

/**
 * @brief Boundary layer solver driver. Performs the space marching and set up time marching for each point.
 * @authors Paul Dechamps
 */
class VII_API Driver : public fwk::wSharedObject
{
public:
    double Cdt;                                         ///< Total drag coefficient.
    double Cdf;                                         ///< Total friction drag coefficient.
    double Cdp;                                         ///< Total pressure drag coefficient.
    std::vector<double> Cdt_sec;                        ///< Sectional total drag coefficient.
    std::vector<double> Cdf_sec;                        ///< Sectional friction drag coefficient.
    std::vector<double> Cdp_sec;                        ///< Sectional pressure drag coefficient.
    std::vector<std::vector<BoundaryLayer *>> sections; ///< Boundary layer regions sorted by section.
    unsigned int verbose;                               ///< Verbosity level of the solver 0<=verbose<=1.

private:
    double Re;                                                       ///< Freestream Reynolds number.
    double CFL0;                                                     ///< Initial CFL number for pseudo time integration.
    double span;                                                     ///< Wing Span (Used only for drag computation, not used if 2D case).
    std::vector<bool> stagPtMvmt;                                    ///< Flag to account for stagnation point movements.
    Solver *tSolver;                                                 ///< Pseudo-time solver.
    int solverExitCode;                                              ///< Exit code of viscous calculations.
    std::vector<std::vector<std::vector<size_t>>> convergenceStatus; ///< Vector containing points that did not converged.

protected:
    fwk::Timers tms; ///< Internal timers

public:
    Driver(double _Re, double _Minf, double _CFL0, size_t nSections, double _span = 0., unsigned int verbose = 1);
    ~Driver();
    int run(unsigned int const couplIter);

    // Getters.
    double getxtr(size_t iSec, size_t iRegion) const { return sections[iSec][iRegion]->xtr; };
    double getRe() const { return Re; };
    std::vector<std::vector<double>> getSolution(size_t iSec);

    // Setters.
    void setGlobMesh(size_t iSec, size_t iRegion, std::vector<double> _x, std::vector<double> _y, std::vector<double> _z);
    void setVelocities(size_t iSec, size_t iRegion, std::vector<double> _vx, std::vector<double> _vy, std::vector<double> _vz);
    void setMe(size_t iSec, size_t iRegion, std::vector<double> _Me);
    void setRhoe(size_t iSec, size_t iRegion, std::vector<double> _rhoe);
    void setxxExt(size_t iSec, size_t iRegion, std::vector<double> _xxExt);
    void setDeltaStarExt(size_t iSec, size_t iRegion, std::vector<double> _deltaStarExt);
    void setUeOld(size_t iSec, size_t iRegion, std::vector<double> _ue);

    // Extractors.
    std::vector<double> extractBlowingVel(size_t iSec, size_t iRegion) const;
    std::vector<double> extractxx(size_t iSec, size_t iRegion) const;
    std::vector<double> extractDeltaStar(size_t iSec, size_t iRegion) const;
    std::vector<double> extractxCoord(size_t iSec, size_t iRegion) const;
    std::vector<double> extractUe(size_t iSec, size_t iRegion) const;

private:
    void checkWaves(size_t iPoint, BoundaryLayer *bl);
    void averageTransition(size_t iPoint, BoundaryLayer *bl, int forced);
    void computeSectionalDrag(std::vector<BoundaryLayer *> const &bl, size_t i);
    void computeTotalDrag();
    void computeBlwVel();
    int outputStatus(double maxMach) const;
};
} // namespace vii
#endif // DDRIVER_H
